#!/usr/bin/env python

import rospy			#for interacting with ROS topics and parameters
import sys, getopt		#for parameters and sys.exit()
from Tkinter import *	#for GUI elements
from std_msgs.msg import Float64MultiArray, MultiArrayDimension
import re				#for findall()
import string 			#for split()

#initialize ROS node
rospy.init_node('GUIControl', anonymous=True)

# initialize Tkinter master
master = Tk()
master.title("Control GUI")

#initialize publisher
goalPublisher = rospy.Publisher(publishTopic, Float64MultiArray)

if (len(sys.argv) > 1): #publish to specified topic
	publishTopic = sys.argv[1]
else: #No argument, publish to joint goal position
	topics = rospy.get_published_topics(namespace = "/")
	for topic in topics:
		robotname = re.findall("/(.+?)/JPosTask/+?",topic[0])
		if robotname !=[]:
			publishTopic = "/" + robotname[0] + "/JPosTask/goalPosition" # set publisher location
			break

print("publishing to " + publishTopic)

#The joint class will create the scale and entry elements in the GUI for each DOF
class Joint:
	jointlist = [] # contains all of the joint elements
	def __init__(self,Label,Min,Max,outFrame):
		self.frame = Frame(outFrame)
		self.scale = Scale(self.frame, from_=Min, to=Max, orient = HORIZONTAL, resolution = 0.05)
		self.scale.config(length = 200, label = Label)
		self.scale.bind()
		self.scale.pack(side = LEFT)
		self.en = Entry(self.frame)
		self.en.bind("<Return>", lambda e: self.setSlider()) #hitting ENTER in the entry box will run setSlider() on the joint
		self.en.config(width = 3)
		self.en.pack(side = LEFT)
		self.frame.pack(side = LEFT)
		Joint.jointlist.append(self)
	def getVal(self):
		return self.scale.get() # will return the scale value
	def setSlider(Joint):
		Joint.scale.set(float(Joint.en.get())) # get the entry value and set the slider to it
		Joint.en.delete(0,END) # clear the entry box

def getArray():
	tempArray=[]
	for el in Joint.jointlist:
		tempArray.append(el.getVal())
	return tempArray

urdf = rospy.get_param("/robot_description")
nameList = re.findall("<joint name=\"(.+?)\">+?",urdf)
filteredNameList=[]
for el in nameList:
	if re.findall("type=\"fixed+?",el) == []:
		filteredNameList.append(el.split("\"",1)[0])
lowBoundList = re.findall(" lower=\"(.+?)\"+?",urdf)
highBoundList = re.findall(" upper=\"(.+?)\"+?",urdf)

index=0
width = input("Please input the number of columns desired:")
while (index < len(filteredNameList)):
	fInd=0
	outFrame = Frame(master)
	while fInd < width and index < len(filteredNameList):
		Joint(filteredNameList[index],float(lowBoundList[index]),float(highBoundList[index]),outFrame)
		fInd+=1
		index+=1
	outFrame.pack()

initArray = getArray()

#create the MultiArray to be published
dim = MultiArrayDimension()
dim.size = len(initArray)
dim.label = "posMsg"
dim.stride = len(initArray)

goalArray = Float64MultiArray()
goalArray.data = initArray
goalArray.layout.dim.append(dim)
goalArray.layout.data_offset = 0

def updateGoal():
	if rospy.is_shutdown():
		sys.exit()
	goalArray.data = getArray()
	goalPublisher.publish(goalArray)
	master.after(50,updateGoal)

master.after(50,updateGoal) # update the goal every 50 ms (20 Hz)
mainloop()